@@ -252,13 +252,18 @@ _pango_get_lc_ctype (void)
 PangoLanguage *
 pango_language_get_default (void)
 {
-  static PangoLanguage *result = NULL;
+  static PangoLanguage *result = NULL; /* MT-safe */
 
-  if (G_UNLIKELY (!result))
+  if (g_once_init_enter (&result))
     {
-      gchar *lang = _pango_get_lc_ctype ();
-      result = pango_language_from_string (lang);
-      g_free (lang);
+      gchar *lc_ctype;
+      PangoLanguage *lang;
+
+      lc_ctype = _pango_get_lc_ctype ();
+      lang = pango_language_from_string (lc_ctype);
+      g_free (lc_ctype);
+
+      g_once_init_leave (&result, lang);
     }
 
   return result;
@@ -288,7 +293,8 @@ pango_language_get_default (void)
 PangoLanguage *
 pango_language_from_string (const char *language)
 {
-  static GHashTable *hash = NULL;
+  G_LOCK_DEFINE_STATIC (lang_from_string);
+  static GHashTable *hash = NULL; /* MT-safe */
   PangoLanguagePrivate *priv;
   char *result;
   int len;
@@ -297,13 +303,15 @@ pango_language_from_string (const char *language)
   if (language == NULL)
     return NULL;
 
+  G_LOCK (lang_from_string);
+
   if (G_UNLIKELY (!hash))
     hash = g_hash_table_new (lang_hash, lang_equal);
   else
     {
       result = g_hash_table_lookup (hash, language);
       if (result)
-	return (PangoLanguage *)result;
+        goto out;
     }
 
   len = strlen (language);
@@ -321,6 +329,9 @@ pango_language_from_string (const char *language)
 
   g_hash_table_insert (hash, result, result);
 
+out:
+  G_UNLOCK (lang_from_string);
+
   return (PangoLanguage *)result;
 }
 
@@ -744,11 +755,14 @@ parse_default_languages (void)
 static PangoLanguage *
 _pango_script_get_default_language (PangoScript script)
 {
-  static gboolean initialized = FALSE;
-  static PangoLanguage * const * languages = NULL;
-  static GHashTable *hash = NULL;
+  G_LOCK_DEFINE_STATIC (languages);
+  static gboolean initialized = FALSE; /* MT-safe */
+  static PangoLanguage * const * languages = NULL; /* MT-safe */
+  static GHashTable *hash = NULL; /* MT-safe */
   PangoLanguage *result, * const * p;
 
+  G_LOCK (languages);
+
   if (G_UNLIKELY (!initialized))
     {
       languages = parse_default_languages ();
@@ -760,10 +774,13 @@ _pango_script_get_default_language (PangoScript script)
     }
 
   if (!languages)
-    return NULL;
+    {
+      result = NULL;
+      goto out;
+    }
 
   if (g_hash_table_lookup_extended (hash, GINT_TO_POINTER (script), NULL, (gpointer *) (gpointer) &result))
-    return result;
+    goto out;
 
   for (p = languages; *p; p++)
     if (pango_language_includes_script (*p, script))
@@ -772,6 +789,9 @@ _pango_script_get_default_language (PangoScript script)
 
   g_hash_table_insert (hash, GINT_TO_POINTER (script), result);
 
+out:
+  G_UNLOCK (languages);
+
   return result;
 }
 
@@ -904,7 +924,34 @@ pango_script_get_sample_language (PangoScript script)
     "",    /* PANGO_SCRIPT_CUNEIFORM */
     "",    /* PANGO_SCRIPT_PHOENICIAN */
     "",    /* PANGO_SCRIPT_PHAGS_PA */
-    "nqo"  /* PANGO_SCRIPT_NKO */
+    "nqo", /* PANGO_SCRIPT_NKO */
+
+    /* Unicode-5.1 additions */
+    "",    /* PANGO_SCRIPT_KAYAH_LI */
+    "",    /* PANGO_SCRIPT_LEPCHA */
+    "",    /* PANGO_SCRIPT_REJANG */
+    "",    /* PANGO_SCRIPT_SUNDANESE */
+    "",    /* PANGO_SCRIPT_SAURASHTRA */
+    "",    /* PANGO_SCRIPT_CHAM */
+    "",    /* PANGO_SCRIPT_OL_CHIKI */
+    "",    /* PANGO_SCRIPT_VAI */
+    "",    /* PANGO_SCRIPT_CARIAN */
+    "",    /* PANGO_SCRIPT_LYCIAN */
+    "",    /* PANGO_SCRIPT_LYDIAN */
+
+    /* Unicode-6.0 additions */
+    "",    /* PANGO_SCRIPT_BATAK */
+    "",    /* PANGO_SCRIPT_BRAHMI */
+    "",    /* PANGO_SCRIPT_MANDAIC */
+
+    /* Unicode-6.1 additions */
+    "",    /* PANGO_SCRIPT_CHAKMA */
+    "",    /* PANGO_SCRIPT_MEROITIC_CURSIVE */
+    "",    /* PANGO_SCRIPT_MEROITIC_HIEROGLYPHS */
+    "",    /* PANGO_SCRIPT_MIAO */
+    "",    /* PANGO_SCRIPT_SHARADA */
+    "",    /* PANGO_SCRIPT_SORA_SOMPENG */
+    "",    /* PANGO_SCRIPT_TAKRI */
   };
   const char *sample_language;
   PangoLanguage *result;
